Unit TextEditor;

INTERFACE

USES
   Windows, Classes, SysUtils, Controls, Graphics, StdCtrls,
   Forms,Messages;

TYPE
//*************************************************************************
   TTextBuffer = Class(TStringList)
   Private
      LineCount: Integer;
      LastLine: Integer;
      LinesPerPage: Integer;
      LineHeight: Integer;
      FirstLine: Integer;
   End;
//*************************************************************************
  TTextEditor = class(TCustomControl)
   Private
      fTextBuffer: TTextBuffer;
      SV: TScrollInfo;

      procedure CreateParams(var Params: TCreateParams); override;
      procedure SetupScrollBars;
   Protected
      procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
   Public
      constructor Create(AOwner: TComponent); Override;
      destructor Destroy; Override;
      Procedure Load(AFileName: String);
      procedure Paint; override;
  End;

IMPLEMENTATION

//*************************************************************************
constructor TTextEditor.Create(AOwner: TComponent);
Begin
  Inherited Create(AOwner);
  fTextBuffer:=TTextBuffer.Create;
End;
//*************************************************************************
destructor TTextEditor.Destroy;
Begin
   fTextBuffer.Free;
   Inherited Destroy;
End;
//*************************************************************************
Procedure TTextEditor.Load(AFileName: String);
Begin
  fTextBuffer.LoadFromFile(AFileName);
  Canvas.Brush.Style:=bsSolid;
  Canvas.Brush.Color:=clWhite;
  Canvas.Font.Color:=clBlack;
  Canvas.Font.Name:='-adobe-courier-medium-r-normal-*-17-*-*-*-*-*-iso10646-1';

  With fTextBuffer Do Begin
     LineCount:= Count;
     LineHeight:= Canvas.TextHeight('A');      // Zeilenhhe
     LinesPerPage:= Height Div LineHeight;     // Zeilen pro Seite
     LastLine:= LineCount-LinesPerPage;        // Letzte Zeile, die
                                          //als erste Zeile fungieren darf
     FirstLine:= 0;                            // Aktuelle erste Zeile
  End;
  SetupScrollBars;
End;
//*************************************************************************
Procedure TTextEditor.Paint;
Var I: Integer;
begin
   With fTextBuffer Do Begin
      I:= 0;
      While (I < LinesPerPage) And (FirstLine+I < LineCount) Do
         Begin
            Canvas.TextOut(0,I*LineHeight,Strings[FirstLine+I]);
            Inc(I);
         End;
   End;
End;
//********************************************************************
procedure TTextEditor.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or WS_VSCROLL;
end;
//*************************************************************************
procedure TTextEditor.SetupScrollBars;
Begin
  if (HandleAllocated) then begin
     SV.cbSize := SizeOf(SV);
     SV.fMask := SIF_POS or SIF_RANGE or SIF_PAGE or SIF_DISABLENOSCROLL;
     SV.nPos := fTextBuffer.FirstLine;
     SV.nMin := 0;
     SV.nMax := fTextBuffer.LineCount;
     SV.nPage:= fTextBuffer.LinesPerPage;
     SetScrollInfo(Handle, SB_Vert, SV, true);
  end
End;
//*************************************************************************
procedure TTextEditor.WMVScroll(var Message: TWMVScroll);
Begin
 inherited;
 With fTextBuffer Do Case Message.ScrollCode of

        SB_TOP: FirstLine:=0;
     SB_BOTTOM: FirstLine:= LastLine;
     SB_LINEUP: If FirstLine > 0
                   Then Dec(FirstLine);
   SB_LINEDOWN: If FirstLine < LastLine
                   Then Inc(FirstLine);
     SB_PAGEUP: If Firstline > LinesPerPage-1
                   Then Dec(FirstLine, LinesPerPage);
   SB_PAGEDOWN: If Firstline < LastLine
                   Then Inc(FirstLine, LinesPerPage);
   SB_THUMBTRACK,SB_THUMBPOSITION:
      begin
          SV.cbSize := SizeOf(SV);
          SV.fMask := SIF_TRACKPOS;
          GetScrollInfo(Handle, SB_Vert, SV);
          fTextBuffer.FirstLine:= SV.nTrackPos;
      end;
   else
      exit;
   end; // Of Case

  SetupScrollBars;
  Refresh;
End;
//*************************************************************************
End.
